/*
 * MikroSDK - MikroE Software Development Kit
 * Copyright (c) 2019, MikroElektronika - www.mikroe.com
 * All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

/*!
 * \file eeprom.c
 *
 */

#include "eeprom.h"


// ------------------------------------------------------------- PRIVATE MACROS

#define EEPROM_DEFAULT_SLAVE_ADDR  0x50

#define EEPROM_WRITE_ENABLE   0
#define EEPROM_WRITE_PROTECT  1

// ------------------------------------------------ PUBLIC FUNCTION DEFINITIONS

void eeprom_cfg_setup( eeprom_cfg_t *cfg )
{
    cfg->wp  = HAL_PIN_NC;
    cfg->scl = HAL_PIN_NC;
    cfg->sda = HAL_PIN_NC;

    cfg->addr_sel = EEPROM_I2C_ADDR_SEL_1;
}

eeprom_retval_t eeprom_init( eeprom_t *context, eeprom_cfg_t *cfg )
{
    i2c_master_config_t i2c_config;

    i2c_master_configure_default( &i2c_config );

    i2c_config.speed    = I2C_STANDARD_SPEED;
    i2c_config.pins.scl = cfg->scl;
    i2c_config.pins.sda = cfg->sda;

    if (i2c_master_open( &context->i2c, &i2c_config ) != I2C_SUCCESS)
    {
        return EEPROM_RETVAL_ERR_DRV_INIT;
    }

    digital_out_init( &context->wp, cfg->wp );
    digital_out_write( &context->wp, EEPROM_WRITE_ENABLE );

    context->slave_addr = EEPROM_DEFAULT_SLAVE_ADDR | cfg->addr_sel;

    return EEPROM_RETVAL_OK;
}

void eeprom_write_byte( eeprom_t *context, uint8_t reg_addr, eeprom_i2c_data_t data_in )
{
    i2c_master_transfer_t transfer;
    eeprom_i2c_data_t buff_data[ 2 ];

    buff_data[ 0 ] = reg_addr;
    buff_data[ 1 ] = data_in;

    transfer.slave_address = context->slave_addr;
    transfer.buffer        = buff_data;
    transfer.count         = 2;
    transfer.end_mode      = I2C_STOP_MODE;

    i2c_master_start( &context->i2c );
    i2c_master_write( &context->i2c, &transfer );
}

void eeprom_write_page( eeprom_t *context, uint8_t reg_addr, eeprom_i2c_data_t *data_in )
{
    i2c_master_transfer_t transfer;
    eeprom_i2c_data_t buff_data[ 17 ];
    uint8_t cnt;

    buff_data[ 0 ] = reg_addr;

    for (cnt = 1; cnt < 17; cnt++)
    {
        buff_data[ cnt ] = *data_in;
        data_in++;
    }

    transfer.slave_address = context->slave_addr;
    transfer.buffer        = buff_data;
    transfer.count         = EEPROM_NBYTES_PAGE + 1;
    transfer.end_mode      = I2C_STOP_MODE;

    i2c_master_start( &context->i2c );
    i2c_master_write( &context->i2c, &transfer );
}

eeprom_i2c_data_t eeprom_read_byte( eeprom_t *context, uint8_t reg_addr )
{
    i2c_master_transfer_t transfer;
    eeprom_i2c_data_t data_;

    data_ = reg_addr;

    transfer.slave_address = context->slave_addr;
    transfer.buffer        = &data_;
    transfer.count         = 1;
    transfer.end_mode      = I2C_RESTART_MODE;

    i2c_master_start( &context->i2c );
    i2c_master_write( &context->i2c, &transfer );

    transfer.end_mode      = I2C_STOP_MODE;

    i2c_master_read( &context->i2c, &transfer );

    return data_;
}

eeprom_retval_t eeprom_read_sequential( eeprom_t *context, uint8_t reg_addr, uint16_t n_bytes, eeprom_i2c_data_t *data_out )
{
    i2c_master_transfer_t transfer;
    eeprom_i2c_data_t data_;

    if ((n_bytes < EEPROM_NBYTES_MIN) || (n_bytes > EEPROM_NBYTES_MAX))
    {
        return EEPROM_RETVAL_ERR_NBYTES;
    }

    data_ = reg_addr;

    transfer.slave_address = context->slave_addr;
    transfer.buffer        = &data_;
    transfer.count         = 1;
    transfer.end_mode      = I2C_RESTART_MODE;

    i2c_master_start( &context->i2c );
    i2c_master_write( &context->i2c, &transfer );

    transfer.buffer        = data_out;
    transfer.count         = n_bytes;
    transfer.end_mode      = I2C_STOP_MODE;

    i2c_master_read( &context->i2c, &transfer );

    return EEPROM_RETVAL_OK;
}

void eeprom_write_enable( eeprom_t *context )
{
    digital_out_write( &context->wp, EEPROM_WRITE_ENABLE );
}

void eeprom_write_protect( eeprom_t *context )
{
    digital_out_write( &context->wp, EEPROM_WRITE_PROTECT );
}

// ------------------------------------------------------------------------ END
